Stan Math Library: stan/math/prim/scal/prob/lognormal_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

lognormal_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_LOGNORMAL_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_LOGNORMAL_LOG_HPP
3
4 #include <boost/random/lognormal_distribution.hpp>
5 #include <boost/random/variate_generator.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_finite.hpp>
9 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
10 #include <stan/math/prim/scal/err/check_not_nan.hpp>
11 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
12 #include <stan/math/prim/scal/fun/constants.hpp>
13 #include <stan/math/prim/scal/fun/value_of.hpp>
14 #include <stan/math/prim/scal/fun/square.hpp>
15 #include <stan/math/prim/scal/meta/length.hpp>
16 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
17 #include <stan/math/prim/scal/meta/contains_nonconstant_struct.hpp>
18 #include <stan/math/prim/scal/meta/VectorView.hpp>
19 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
20 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
21 #include <stan/math/prim/scal/meta/return_type.hpp>
22 #include <stan/math/prim/scal/meta/include_summand.hpp>
23 #include <cmath>
24
25 namespace stan {
26 namespace math {
27
28 // LogNormal(y|mu, sigma) [y >= 0; sigma > 0]
29 // FIXME: document
30 template <bool propto,
31 typename T_y, typename T_loc, typename T_scale>
32 typename return_type<T_y, T_loc, T_scale>::type
33 lognormal_log(const T_y& y, const T_loc& mu, const T_scale& sigma) {
34 static const char* function("stan::math::lognormal_log");
35 typedef typename stan::partials_return_type<T_y, T_loc, T_scale>::type
36 T_partials_return;
37
38 using stan::is_constant_struct;
39 using stan::math::check_not_nan;
40 using stan::math::check_finite;
41 using stan::math::check_positive_finite;
42 using stan::math::check_nonnegative;
43 using stan::math::check_consistent_sizes;
44 using stan::math::value_of;
45 using stan::math::include_summand;
46
47
48 // check if any vectors are zero length
49 if (!(stan::length(y)
50 && stan::length(mu)
51 && stan::length(sigma)))
52 return 0.0;
53
54 // set up return value accumulator
55 T_partials_return logp(0.0);
56
57 // validate args (here done over var, which should be OK)
58 check_not_nan(function, "Random variable", y);
59 check_nonnegative(function, "Random variable", y);
60 check_finite(function, "Location parameter", mu);
61 check_positive_finite(function, "Scale parameter", sigma);
62 check_consistent_sizes(function,
63 "Random variable", y,
64 "Location parameter", mu,
65 "Scale parameter", sigma);
66
67 VectorView<const T_y> y_vec(y);
68 VectorView<const T_loc> mu_vec(mu);
69 VectorView<const T_scale> sigma_vec(sigma);
70 size_t N = max_size(y, mu, sigma);
71
72 for (size_t n = 0; n < length(y); n++)
73 if (value_of(y_vec[n]) <= 0)
74 return LOG_ZERO;
75
76 OperandsAndPartials<T_y, T_loc, T_scale>
77 operands_and_partials(y, mu, sigma);
78
79 using stan::math::square;
80 using std::log;
81 using stan::math::NEG_LOG_SQRT_TWO_PI;
82 using std::log;
83
84
85 VectorBuilder<include_summand<propto, T_scale>::value,
86 T_partials_return, T_scale> log_sigma(length(sigma));
87 if (include_summand<propto, T_scale>::value) {
88 for (size_t n = 0; n < length(sigma); n++)
89 log_sigma[n] = log(value_of(sigma_vec[n]));
90 }
91
92 VectorBuilder<include_summand<propto, T_y, T_loc, T_scale>::value,
93 T_partials_return, T_scale> inv_sigma(length(sigma));
94 VectorBuilder<include_summand<propto, T_y, T_loc, T_scale>::value,
95 T_partials_return, T_scale> inv_sigma_sq(length(sigma));
96 if (include_summand<propto, T_y, T_loc, T_scale>::value) {
97 for (size_t n = 0; n < length(sigma); n++)
98 inv_sigma[n] = 1 / value_of(sigma_vec[n]);
99 }
100 if (include_summand<propto, T_y, T_loc, T_scale>::value) {
101 for (size_t n = 0; n < length(sigma); n++)
102 inv_sigma_sq[n] = inv_sigma[n] * inv_sigma[n];
103 }
104
105 VectorBuilder<include_summand<propto, T_y, T_loc, T_scale>::value,
106 T_partials_return, T_y> log_y(length(y));
107 if (include_summand<propto, T_y, T_loc, T_scale>::value) {
108 for (size_t n = 0; n < length(y); n++)
109 log_y[n] = log(value_of(y_vec[n]));
110 }
111
112 VectorBuilder<!is_constant_struct<T_y>::value,
113 T_partials_return, T_y> inv_y(length(y));
114 if (!is_constant_struct<T_y>::value) {
115 for (size_t n = 0; n < length(y); n++)
116 inv_y[n] = 1 / value_of(y_vec[n]);
117 }
118
119 if (include_summand<propto>::value)
120 logp += N * NEG_LOG_SQRT_TWO_PI;
121
122 for (size_t n = 0; n < N; n++) {
123 const T_partials_return mu_dbl = value_of(mu_vec[n]);
124
125 T_partials_return logy_m_mu(0);
126 if (include_summand<propto, T_y, T_loc, T_scale>::value)
127 logy_m_mu = log_y[n] - mu_dbl;
128
129 T_partials_return logy_m_mu_sq = logy_m_mu * logy_m_mu;
130 T_partials_return logy_m_mu_div_sigma(0);
131 if (contains_nonconstant_struct<T_y, T_loc, T_scale>::value)
132 logy_m_mu_div_sigma = logy_m_mu * inv_sigma_sq[n];
133
134
135 // log probability
136 if (include_summand<propto, T_scale>::value)
137 logp -= log_sigma[n];
138 if (include_summand<propto, T_y>::value)
139 logp -= log_y[n];
140 if (include_summand<propto, T_y, T_loc, T_scale>::value)
141 logp -= 0.5 * logy_m_mu_sq * inv_sigma_sq[n];
142
143 // gradients
144 if (!is_constant_struct<T_y>::value)
145 operands_and_partials.d_x1[n] -= (1 + logy_m_mu_div_sigma) * inv_y[n];
146 if (!is_constant_struct<T_loc>::value)
147 operands_and_partials.d_x2[n] += logy_m_mu_div_sigma;
148 if (!is_constant_struct<T_scale>::value)
149 operands_and_partials.d_x3[n]
150 += (logy_m_mu_div_sigma * logy_m_mu - 1) * inv_sigma[n];
151 }
152 return operands_and_partials.value(logp);
153 }
154
155 template <typename T_y, typename T_loc, typename T_scale>
156 inline
157 typename return_type<T_y, T_loc, T_scale>::type
158 lognormal_log(const T_y& y, const T_loc& mu, const T_scale& sigma) {
159 return lognormal_log<false>(y, mu, sigma);
160 }
161 }
162 }
163 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
square.hpp
contains_nonconstant_struct.hpp
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::LOG_ZERO
const double LOG_ZERO
Definition: constants.hpp:175
return_type.hpp
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
stan::math::square
fvar< T > square(const fvar< T > &x)
Definition: square.hpp:15
include_summand.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::math::lognormal_log
return_type< T_y, T_loc, T_scale >::type lognormal_log(const T_y &y, const T_loc &mu, const T_scale &sigma)
Definition: lognormal_log.hpp:33
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::NEG_LOG_SQRT_TWO_PI
const double NEG_LOG_SQRT_TWO_PI
Definition: constants.hpp:184
stan::contains_nonconstant_struct
Definition: contains_nonconstant_struct.hpp:14
length.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39

[ Stan Home Page ] © 2011–2016, Stan Development Team.